package exercises;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Random;

import static java.lang.System.*;

/**
 * Catching raindrops in bucket (possibly a game?)
 */
public class CatchTheRain extends JPanel {

    public static void main(String[] args) {
        new CatchTheRain().program();
    }

    final int width = 500;
    final int height = 500;

    final Random rand = new Random();
    final Bucket bucket = new Bucket(width / 2, 40, 40, 40, 4, Color.BLUE);
    final RainDrop[] drops = new RainDrop[10];

    void program() {
        // Testing
        /*Bucket b = new Bucket(15, 15 , 10 , 10 , 0 , null);
        RainDrop d = new RainDrop(10, 10 , 10 , 0 , null);
        Rectangle bb = b.getBoundingBox();
        Rectangle db = d.getBoundingBox();
        out.println( bb.intersects(db));*/

        for (int i = 0; i < drops.length; i++) {
            drops[i] = createRainDrop();
        }
        initGraphics();
        initEvents();
    }

    // ---- Game logic ---------

    void updateWorld() {
        for (int i = 0; i < drops.length; i++) {
            if (drops[i] != null) {
                drops[i].move();
                if (hitFloor(drops[i]) || hitBucket(bucket, drops[i])) {
                    drops[i] = createRainDrop();

                }
                // If hit bucket add points
            }
        }
    }

    boolean hitBucket(Bucket b, RainDrop d) {
        // Must use flip bounding boxes else view of
        // Collision will not match logical collision
        Rectangle br = b.getBoundingBox();
        Rectangle dr = d.getBoundingBox();
        return flipBoundingBox(br).intersects(flipBoundingBox(dr));
    }

    boolean hitFloor(RainDrop d) {
        return d.getBoundingBox().y < 0;
    }

    void handleKeys(int keyCode) {
        if (keyCode == KeyEvent.VK_LEFT) {
            if (0 < bucket.x - bucket.width) {
                bucket.left();
            }
        } else if (keyCode == KeyEvent.VK_RIGHT) {
            if (bucket.x + bucket.width < width) {
                bucket.right();
            }
        }
    }

    // -- Utilities -----------------------

    RainDrop createRainDrop() {
        Color c = new Color(rand.nextInt(255), rand.nextInt(255), rand.nextInt(255));
        return new RainDrop(rand.nextInt(width), height + 50,
                rand.nextInt(10) + 2, rand.nextInt(3) + 1, c);
    }

    // Convert logical bounding box too screen coordinates
    Rectangle flipBoundingBox(Rectangle r) {
        r.y = height - r.y;
        return r;
    }
    //----- Classes --------------------------------

    class RainDrop {
        final double x;     // x and y for center
        double y;
        final double dy;    // Speed
        final double radius;
        final Color color;

        RainDrop(double x, double y, double radius, double dy, Color color) {
            this.x = x;
            this.y = y;
            this.dy = dy;
            this.radius = radius;
            this.color = color;
        }

        void move() {
            y -= dy;
        }

        Rectangle getBoundingBox() {
            int x = (int) (this.x - radius);
            int y = (int) (this.y + radius);
            int h = (int) (2 * radius);
            int w = (int) (2 * radius);
            return new Rectangle(x, y, w, h);
        }
    }

    class Bucket {
        final Color color;
        final double dx;
        final double width;
        final double height;
        double x;   // Left
        final double y;   // Top

        Bucket(double x, double y, double width, double height, double dx, Color color) {
            this.x = x;
            this.y = y;
            this.dx = dx;
            this.width = width;
            this.height = height;
            this.color = color;
        }

        void right() {
            x += dx;
        }

        void left() {
            x -= dx;
        }

        Rectangle getBoundingBox() {
            int x = (int) (this.x);
            int y = (int) (this.y);
            int h = (int) height;
            int w = (int) width;
            return new Rectangle(x, y, w, h);
        }
    }

    // ---- Below is just initialization and graphics (i.e. no program logic) ------------
    final int delay = 20;

    public void paint(Graphics g) {
        super.paint(g);
        Graphics2D g2 = (Graphics2D) g;
        drawBucket(g2, bucket);
        drawRainDrops(g2, drops);
        Toolkit.getDefaultToolkit().sync();
    }

    // Draw all drops
    void drawRainDrops(Graphics2D g2, RainDrop[] drops) {
        for (int i = 0; i < drops.length; i++) {
            if (drops[i] != null) {
                drawRainDrop(g2, drops[i]);
            }
        }
    }

    void drawBucket(Graphics2D g2, Bucket b) {
        Color old = g2.getColor();
        g2.setColor(b.color);
        g2.fillRect((int) (b.x), (int) (height - (b.y)),
                (int) (b.width), (int) (b.height));
        g2.setColor(old);
    }

    // A single drop
    void drawRainDrop(Graphics2D g2, RainDrop d) {
        Color old = g2.getColor();
        //Rectangle r = d.getBoundingBox();
        //g2.drawRect(r.x, height - r.y, r.width, r.height);
        g2.setColor(d.color);
        g2.fillOval((int) (d.x - d.radius), (int) (height - (d.y + d.radius)),
                (int) (2 * d.radius), (int) (2 * d.radius));
        g2.setColor(old);
    }

    void initGraphics() {
        setPreferredSize(new Dimension(width, height));
        JFrame window = new JFrame();
        window.setTitle("Catch the Rain");
        window.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        window.add(this);
        window.pack();
        window.setLocationRelativeTo(null);
        window.setVisible(true);
        window.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                handleKeys(e.getKeyCode());
            }
        });
    }

    void initEvents() {
        Timer timer = new Timer(delay, e -> {
            updateWorld();
            repaint();
        });
        timer.start();
    }
}
